from aiharness.configuration import Arguments
from transformersx.configuration import DownloadConfiguration, log
from transformersx.models import download_models


def test_task(config: DownloadConfiguration):
    '''
        a task for test
    '''
    log.info('Run the test task with configuration: ' + str(config))


TASKS = {
    'test': test_task,
    'download_models': download_models
}

def task_names():
    return TASKS.keys()


def get_task(task_name):
    return TASKS.get(task_name)


def main():
    config: DownloadConfiguration = Arguments(DownloadConfiguration()).parse()

    task_name = config.task_name
    task = get_task(task_name)
    if not task:
        log.error('Please specify the corrent task name, it should be %s, but the input is %s' % (
            str(task_names()), task_name))
        return

    task(config)


if __name__ == "__main__":
    main()



import os
import random
import sys

import numpy as np
import torch
from aiharness.fileutils import join_path
from transformers import HfArgumentParser, AutoConfig, AutoTokenizer, Trainer

from transformersx.configuration import ModelArguments, DataTrainingArguments,log
from transformers.training_args import TrainingArguments
from transformersx import models

def parse_args():
    parser = HfArgumentParser((ModelArguments, DataTrainingArguments, TrainingArguments))

    if len(sys.argv) == 2 and sys.argv[1].endswith(".json"):
        # If we pass only one argument to the script and it's the path to a json file,
        # let's parse it to get our arguments.
        return parser.parse_json_file(json_file=os.path.abspath(sys.argv[1]))
    else:
        return parser.parse_args_into_dataclasses()


def set_seed(seed: int):
    random.seed(seed)
    np.random.seed(seed)
    torch.manual_seed(seed)
    torch.cuda.manual_seed_all(seed)
    # ^^ safe to call this function even if cuda is not available


class TaskModel:
    def __init__(self, modelArgs: ModelArguments, model_class=None):
        self._model_args = modelArgs
        self.model_path = modelArgs.model_path
        self._model_class = model_class
        self.config, self.tokenizer, self.model = self._init()

    def _init(self):
        config = AutoConfig.from_pretrained(
            self._model_args.model_path,
            num_labels=self._model_args.num_labels
        )

        tokenizer = AutoTokenizer.from_pretrained(
            self._model_args.model_path
        )

        model = self._model(config)

        return config, tokenizer, model

    def _model(self, config):
        mode_path = join_path(self._model_args.model_base_dir,
                              self._model_args.model_size,
                              self._model_args.model_path)

        model_cls = self._model_class if self._model_class else models.model_class(self.config)
        return model_cls.from_pretrained(
            mode_path,
            config=config
        )


class TaskData:
    def __init__(self, dataArgs: DataTrainingArguments,
                 tokenizer, datasetClass, local_rank):
        self._data_args = dataArgs
        self._tokenizer = tokenizer
        self._datasetClass = datasetClass
        self._local_rank = local_rank
        self._train_data, self._eval_data = None, None

    def train_data(self):
        if not self._train_data:
            self._train_data = self._datasetClass(self._data_args,
                                                  tokenizer=self._tokenizer,
                                                  local_rank=self._local_rank)
        return self._train_data

    def eval_data(self):
        if not self._eval_data:
            self._eval_data = self._datasetClass(self._data_args,
                                                 tokenizer=self._tokenizer,
                                                 local_rank=self._local_rank,
                                                 evaluate=True)
        return self._eval_data


class TaskTrainer:
    def __init__(self, trainingArgs: TrainingArguments,
                 taskModel: TaskModel, taskData: TaskData, compute_metrics=None):
        self._training_args = trainingArgs
        self._taskModel = taskModel
        self._taskData = taskData
        self._compute_metrics = compute_metrics
        self._trainer = self._create_trainer()

    def _check_train_args(self):
        if (
                os.path.exists(self._training_args.output_dir)
                and os.listdir(self._training_args.output_dir)
                and self._training_args.do_train
                and not self._training_args.overwrite_output_dir
        ):
            raise ValueError(
                f"Output directory ({self._training_args.output_dir}) already exists and is not empty. Use --overwrite_output_dir to overcome."
            )

    def _create_trainer(self):
        return Trainer(
            model=self._taskModel.model,
            args=self._training_args,
            train_dataset=self._taskData.train_data() if self._training_args.do_train else None,
            eval_dataset=self._taskData.eval_data() if self._training_args.do_eval else None,
            compute_metrics=self._compute_metrics,
        )

    def train(self):
        if self._training_args.do_train:
            self._trainer.train(
                model_path=self._taskModel.model_path
            )
            self._trainer.save_model()
            # For convenience, we also re-save the tokenizer to the same directory,
            # so that you can share your model easily on huggingface.co/models =)
            if self._trainer.is_world_master():
                self._taskModel.tokenizer.save_pretrained(self._training_args.output_dir)
        return self

    def eval(self):
        # Evaluation
        results = {}
        if self._training_args.do_eval and self._training_args.local_rank in [-1, 0]:
            log.info("*** Evaluate ***")

            eval_datasets = [self._taskData.eval_data()]

            for eval_dataset in eval_datasets:
                result = self._trainer.evaluate(eval_dataset=eval_dataset)

                output_eval_file = join_path(
                    self._training_args.output_dir, f"eval_results_{eval_dataset.args.task_name}.txt"
                )
                with open(output_eval_file, "w") as writer:
                    log.info("***** Eval results {} *****".format(eval_dataset.args.task_name))
                    for key, value in result.items():
                        log.info("  %s = %s", key, value)
                        writer.write("%s = %s\n" % (key, value))

                results.update(result)
        return results


class Task:
    def __init__(self,
                 modelArgs: ModelArguments,
                 dataArgs: DataTrainingArguments,
                 trainingArgs: TrainingArguments,
                 datasetClass, model_class=None, compute_metric=None):
        self._taskModel = TaskModel(modelArgs, model_class)
        self._taskData = TaskData(dataArgs, self._taskModel.tokenizer, datasetClass, trainingArgs.local_rank)
        self._taskTrainer = TaskTrainer(trainingArgs, self._taskModel, self._taskData, compute_metric)
        self._training_args = trainingArgs

    def _log_task_start(self):
        log.warning(
            "Process rank: %s, device: %s, n_gpu: %s, distributed training: %s, 16-bits training: %s",
            self._training_args.local_rank,
            self._training_args.device,
            self._training_args.n_gpu,
            bool(self._training_args.local_rank != -1),
            self._training_args.fp16,
        )
        log.info("Training/evaluation parameters %s", self._training_args)

    def start(self):
        self._log_task_start()

        set_seed(self._training_args.seed)
        results = self._taskTrainer.train().eval()
        return results



class DataProcessor:
    """Base class for data converters for sequence classification data sets."""

    def get_train_examples(self, data_dir):
        """Gets a collection of `InputExample`s for the train set."""
        raise NotImplementedError()

    def get_dev_examples(self, data_dir):
        """Gets a collection of `InputExample`s for the dev set."""
        raise NotImplementedError()

    def get_labels(self):
        """Gets the list of labels for this data set."""
        raise NotImplementedError()



from transformers.modeling_auto import MODEL_FOR_MULTIPLE_CHOICE_MAPPING, \
    MODEL_FOR_PRETRAINING_MAPPING, MODEL_FOR_QUESTION_ANSWERING_MAPPING, MODEL_FOR_SEQUENCE_CLASSIFICATION_MAPPING, \
    MODEL_FOR_TOKEN_CLASSIFICATION_MAPPING, MODEL_MAPPING, MODEL_WITH_LM_HEAD_MAPPING
from transformersx.configuration import DownloadConfiguration
from transformers.modeling_auto import AutoConfig, AutoModel
from transformers.tokenization_auto import AutoTokenizer
from aiharness.executors import QueueExecutor
from aiharness import harnessutils as utils
from aiharness.fileutils import join_path

log = utils.getLogger('task')

all_model_mappings = [MODEL_MAPPING,
                      MODEL_WITH_LM_HEAD_MAPPING,
                      MODEL_FOR_TOKEN_CLASSIFICATION_MAPPING,
                      MODEL_FOR_SEQUENCE_CLASSIFICATION_MAPPING,
                      MODEL_FOR_QUESTION_ANSWERING_MAPPING,
                      MODEL_FOR_PRETRAINING_MAPPING,
                      MODEL_FOR_MULTIPLE_CHOICE_MAPPING
                      ]


def model_class(config):
    for mapping in all_model_mappings:
        for config_class, model_class in mapping.items():
            if isinstance(config, config_class):
                return model_class


distil_models = [
    'adamlin/bert-distil-chinese',
]

tiny_models = [
    'clue/albert_chinese_tiny',
    'voidful/albert_chinese_tiny',
    'clue/roberta_chinese_3L312_clue_tiny',
    'clue/roberta_chinese_3L768_clue_tiny',
    'clue/roberta_chinese_clue_tiny',
    'clue/roberta_chinese_pair_tiny',
    'lonePatient/roberta_chinese_clue_tiny'
]

small_models = [
    'clue/albert_chinese_small',
    'voidful/albert_chinese_small',
    'lonePatient/albert_chinese_small',
    'hfl/chinese-electra-small-discriminator',
    'hfl/chinese-electra-small-generator'
]

base_models = [
    'bert-base-chinese',
    'clue/roberta_chinese_base',
    'hfl/chinese-roberta-wwm-ext',
    'voidful/albert_chinese_base',
    'hfl/chinese-bert-wwm-ext',
    'hfl/chinese-bert-wwm',
    'hfl/chinese-electra-base-discriminator',
    'hfl/chinese-electra-base-generator',
    'hfl/chinese-xlnet-base',
    'hfl/chinese-xlnet-mid',

]

large_models = [
    'clue/roberta_chinese_large',
    'clue/xlnet_chinese_large',
    'voidful/albert_chinese_large',
    'voidful/albert_chinese_xlarge',
    'voidful/albert_chinese_xxlarge',
    'clue/roberta_chinese_clue_large',
    'clue/roberta_chinese_pair_large',
    'hfl/chinese-roberta-wwm-ext-large'
]


class Downloader():
    def __init__(self, config: DownloadConfiguration):
        self._config = config
        model_size = config.model_size
        self.models = eval(model_size + '_models')

    def __call__(self, *args, **kwargs):
        QueueExecutor(self.models, worker_num=8).run(self._download_model)

    def _download_model(self, model_names):
        for model_name in model_names:
            log.info('Initial model of ' + model_name)
            cache_dir = join_path(self._config.cache_dir, model_name)
            AutoConfig.from_pretrained(model_name, cache_dir=cache_dir)
            AutoTokenizer.from_pretrained(model_name, cache_dir=cache_dir)
            AutoModel.from_pretrained(model_name, cache_dir=cache_dir)


def download_models(config: DownloadConfiguration):
    Downloader(config)()



from typing import Optional

from aiharness.configuration import field, configclass
from dataclasses import dataclass
from aiharness import harnessutils as aiutils

log = aiutils.getLogger('task')

@configclass()
class DownloadConfiguration:
    model: str = field('electra', 'specified the model')
    model_size: str = field('base', 'specifiy the model size')
    cache_dir: str = field('nlp_models_cache', 'specified the cache dir for models')
    task_name: str = field('download_models', 'specified the task name')


@dataclass
class ModelArguments:
    """
    Arguments pertaining to which model/config/tokenizer we are going to fine-tune from.
    """

    model_path: str = field(
        metadata={"help": "Path to pretrained model or model identifier from huggingface.co/models"}
    )
    config_name: Optional[str] = field(
        default=None, metadata={"help": "Pretrained config name or path if not the same as model_name"}
    )
    tokenizer_name: Optional[str] = field(
        default=None, metadata={"help": "Pretrained tokenizer name or path if not the same as model_name"}
    )
    cache_dir: Optional[str] = field(
        default=None, metadata={"help": "Where do you want to store the pretrained models downloaded from s3"}
    )

    model_base_dir: str = field(default="",
                                metadata={"help": "the path base dir of models"})

    model_mode: str = field(default=0,
                            metadata={"help": "the model of model: classification or regression"})

    model_size: str = field(default="base",
                            metadata={"help": "the size of model: tiny,small,base,large"})

    num_labels: str = field(default=2,
                            metadata={"help": "the number of label"})


@dataclass
class DataTrainingArguments:
    """
    Arguments pertaining to what data we are going to input our model for training and eval.

    Using `HfArgumentParser` we can turn this class
    into argparse arguments to be able to specify them on
    the command line.
    """
    task_name: str = field(default="",
                           metadata={
                               "help": "The input data dir. Should contain the .tsv files (or other data files) for the task."}
                           )

    model_mode: str = field(default=0,
                            metadata={"help": "the model of model: classification or regression"})

    data_dir: str = field(
        metadata={"help": "The input data dir. Should contain the .tsv files (or other data files) for the task."}
    )
    max_seq_length: int = field(
        default=128,
        metadata={
            "help": "The maximum total input sequence length after tokenization. Sequences longer "
                    "than this will be truncated, sequences shorter will be padded."
        },
    )
    overwrite_cache: bool = field(
        default=False, metadata={"help": "Overwrite the cached training and evaluation sets"}
    )
